/* GtkTetcolor
 * Copyright (C) 1999 Andrey V. Panov
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdlib.h>
#include <gdk/gdkkeysyms.h>
#include <stdio.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "main.h"
#include "score.h"

gint cell_width, left_margin, initial_level, use_graykeys;
gint nav_keys[4], alt_nav_keys[4];

static const gchar *MainWindowKey = "MainWindowKey";

gboolean preferences_changed;
gboolean preferences_dialog_shown = FALSE, about_dialog_shown = FALSE;
gboolean scores_dialog_shown = FALSE, help_dialog_shown = FALSE;


gboolean
on_main_window_delete_event (GtkWidget * widget,
			     GdkEvent * event, gpointer user_data)
{
  gtk_exit (0);
  return FALSE;
}


void
on_New_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  init_game (GTK_WIDGET (menuitem));
}


void
on_Scores_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *dialog;

  if (scores_dialog_shown)
    return;
  scores_dialog_shown = TRUE;
  dialog = create_scores_dialog ();
  gtk_widget_show (dialog);
}


void
on_Quit_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  gtk_exit (0);
}


void
on_About_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *about;

  if (about_dialog_shown)
    return;
  about_dialog_shown = TRUE;
  about = create_about_dialog ();
  gtk_widget_show (about);
}


void
on_new_button_clicked (GtkButton * button, gpointer user_data)
{
  init_game (GTK_WIDGET (button));
}


void
on_scores_button_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *dialog;

  if (scores_dialog_shown)
    return;
  scores_dialog_shown = TRUE;
  dialog = create_scores_dialog ();
  gtk_widget_show (dialog);
}


void
on_pause_button_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *dialog, *main_window;

  if (continue_game) {
    main_window = lookup_widget (GTK_WIDGET (button), "main_window");
    dialog = create_pause_dialog ();
    gtk_object_set_data (GTK_OBJECT (dialog), MainWindowKey, main_window);
    if (timeout)
      gtk_timeout_remove (timeout);
    if (level_timeout)
      gtk_timeout_remove (level_timeout);
    continue_game = 0;
    gtk_widget_show (dialog);
  }
}

void
on_exit_button_clicked (GtkButton * button, gpointer user_data)
{
  gtk_exit (0);
}

void
on_ok_about_button_clicked (GtkButton * button, gpointer user_data)
{
  about_dialog_shown = FALSE;
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}

gboolean
on_main_window_key_press_event (GtkWidget * widget,
				GdkEventKey * event, gpointer user_data)
{
  gint key = event->keyval;
  GtkWidget *drawingarea;
  GtkWidget *window;

  drawingarea = lookup_widget (GTK_WIDGET (widget), "drawingarea1");
  window = lookup_widget (GTK_WIDGET (widget), "main_window");
  if (!timeout || !continue_game)
    return TRUE;
  if (key == nav_keys[0] || key == alt_nav_keys[0]){
    if (shift_block (1))
      redraw_cells (drawingarea);}
  else if (key == nav_keys[1] || key == alt_nav_keys[1]){
    if (shift_block (-1))
      redraw_cells (drawingarea);}
  else if (key == nav_keys[2] || key == alt_nav_keys[2]){
    if (block.type)
      if (rotate_block (rotate_cell_left))
	redraw_cells (drawingarea);}
  else if (key == nav_keys[3] || key == alt_nav_keys[3]){
    if (block.type)
      if (rotate_block (rotate_cell_right))
	redraw_cells (drawingarea);}
  else if (key == GDK_space){
    if (timeout) {
      gtk_timeout_remove (timeout);
      interval = INI_INTERVAL / 100;
      timeout = gtk_timeout_add (interval, (GtkFunction) timeout_callback,
				 window);
    }
    else
      return TRUE;
  }
  return FALSE;
}


void
on_scores_dialog_button_clicked (GtkButton * button, gpointer user_data)
{
  scores_dialog_shown = FALSE;
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}


void
on_scores_dialog_show (GtkWidget * widget, gpointer user_data)
{
  GtkWidget *name_label[10], *score_label[10];
  GtkWidget *name_hbox[10], *score_hbox[10];
  GtkWidget *scores_table;
  gint i;
  gchar str[10];

  scores_table = lookup_widget (GTK_WIDGET (widget), "scores_table");
  read_score ();
  for (i = 0; i < 10; i++) {
    name_hbox[i] = gtk_hbox_new (FALSE, 0);
    gtk_widget_ref (name_hbox[i]);
    gtk_object_set_data_full (GTK_OBJECT (scores_table), "", name_hbox[i],
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (name_hbox[i]);

    score_hbox[i] = gtk_hbox_new (FALSE, 0);
    gtk_widget_ref (score_hbox[i]);
    gtk_object_set_data_full (GTK_OBJECT (scores_table), "", score_hbox[i],
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (score_hbox[i]);
    name_label[i] = gtk_label_new (name[i]);
    gtk_widget_ref (name_label[i]);
    gtk_object_set_data_full (GTK_OBJECT (scores_table), "", name_label[i],
			      (GtkDestroyNotify) gtk_widget_unref);
    g_snprintf (str, 10, "%d", saved_score[i]);
    score_label[i] = gtk_label_new (str);
    gtk_widget_ref (score_label[i]);
    gtk_object_set_data_full (GTK_OBJECT (scores_table), "", score_label[i],
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_table_attach_defaults (GTK_TABLE (scores_table), name_hbox[i], 0, 1, i,
			       i + 1);
    gtk_box_pack_start (GTK_BOX (name_hbox[i]), name_label[i], FALSE, FALSE, 0);
    gtk_table_attach_defaults (GTK_TABLE (scores_table), score_hbox[i], 1, 2, i,
			       i + 1);
    gtk_box_pack_end (GTK_BOX (score_hbox[i]), score_label[i], FALSE, FALSE, 0);
    gtk_widget_show (name_label[i]);
    gtk_widget_show (score_label[i]);
  }
}


void
on_name_ok_button_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *widget;
  gchar *str;

  widget = lookup_widget (GTK_WIDGET (button), "name_entry");
  str = gtk_entry_get_text (GTK_ENTRY (widget));
  if (strlen (str) == 0) {
    str = g_get_real_name ();
    if (strlen (str) == 0)
      str = g_get_user_name ();
  }
  strcpy (new_name, str);
  strcpy (name[name_i], new_name);
  saved_score[name_i] = score;
  choose_score_file ();
  write_score ();
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}


void
on_pause_ok_button_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *window, *pause_dialog;

  continue_game = 1;
  pause_dialog = gtk_widget_get_toplevel (GTK_WIDGET (button));
  window = gtk_object_get_data (GTK_OBJECT (pause_dialog), MainWindowKey);
  interval = INI_INTERVAL - INI_INTERVAL * (level - 1) / 10;
  timeout = gtk_timeout_add (interval, (GtkFunction) timeout_callback, window);
  level_timeout =
    gtk_timeout_add (LEVEL_INT, (GtkFunction) change_level, window);
  gtk_widget_destroy (pause_dialog);
}


void
on_pause_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *dialog, *main_window;

  if (continue_game) {
    main_window = lookup_widget (GTK_WIDGET (menuitem), "main_window");
    dialog = create_pause_dialog ();
    gtk_object_set_data (GTK_OBJECT (dialog), MainWindowKey, main_window);
    if (timeout)
      gtk_timeout_remove (timeout);
    if (level_timeout)
      gtk_timeout_remove (level_timeout);
    continue_game = 0;
    gtk_widget_show (dialog);
  }
}


void
on_on_keys_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *dialog;

  if (help_dialog_shown)
    return;
  help_dialog_shown = TRUE;
  dialog = create_help_dialog ();
  gtk_widget_show (dialog);
}


void
on_help_button_clicked (GtkButton * button, gpointer user_data)
{
  help_dialog_shown = FALSE;
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}

gboolean
on_main_window_expose_event (GtkWidget * widget,
			     GdkEventExpose * event, gpointer user_data)
{
  GtkWidget *drawingarea;

  drawingarea = lookup_widget (GTK_WIDGET (widget), "drawingarea1");
  redraw_all_cells (drawingarea);
  return FALSE;
}


void
on_preferences_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *dialog, *main_window;

  if (preferences_dialog_shown)
    return;
  preferences_dialog_shown = TRUE;
  main_window = lookup_widget (GTK_WIDGET (menuitem), "main_window");
  dialog = create_preferences_dialog ();
  gtk_object_set_data (GTK_OBJECT (dialog), MainWindowKey, main_window);
  gtk_widget_show (dialog);
}


void
on_preferences_ok_button_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *preferences_dialog;
  FILE *rcfile;

  preferences_dialog =
    lookup_widget (GTK_WIDGET (button), "preferences_dialog");
  change_preferences (preferences_dialog);
  if (preferences_changed) {
    rcfile = fopen (rcfile_path, "w");
    if (rcfile == NULL)
      g_print ("cannot open rc file for writing");
    else {
      fprintf (rcfile, "cell_size %d\n", cell_width);
      fprintf (rcfile, "initial_level %d\n", initial_level);
      fprintf (rcfile, "use_graykeys %d\n", use_graykeys);
      fclose (rcfile);
    }
  }
  preferences_dialog_shown = FALSE;
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}


void
on_preferences_apply_button_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *preferences_dialog;

  preferences_dialog =
    lookup_widget (GTK_WIDGET (button), "preferences_dialog");
  change_preferences (preferences_dialog);
}


void
on_preferences_close_button_clicked (GtkButton * button, gpointer user_data)
{
  preferences_dialog_shown = FALSE;
  gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}


void
on_preferences_dialog_show (GtkWidget * widget, gpointer user_data)
{
  GtkWidget *cell_size_spinbutton, *initial_level_spinbutton;
  GtkWidget *graykeys_radiobutton;

  preferences_changed = FALSE;
  cell_size_spinbutton =
    lookup_widget (GTK_WIDGET (widget), "cell_size_spinbutton");
  initial_level_spinbutton =
    lookup_widget (GTK_WIDGET (widget), "initial_level_spinbutton");
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (cell_size_spinbutton),
			     (gfloat) cell_width);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (initial_level_spinbutton),
			     (gfloat) initial_level);
  if (use_graykeys){
    graykeys_radiobutton = lookup_widget (GTK_WIDGET (widget), "graykeys_radiobutton");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (graykeys_radiobutton), TRUE);
  }			
}

void
change_preferences (GtkWidget * widget)
{
  GtkWidget *cell_size_spinbutton, *initial_level_spinbutton;
  GtkWidget *window, *drawingarea;
  gint temp;

  cell_size_spinbutton =
    lookup_widget (GTK_WIDGET (widget), "cell_size_spinbutton");
  initial_level_spinbutton =
    lookup_widget (GTK_WIDGET (widget), "initial_level_spinbutton");
  temp =
    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (cell_size_spinbutton));
  if (temp != cell_width) {
    cell_width = temp;
    window = gtk_object_get_data (GTK_OBJECT (widget), MainWindowKey);
    drawingarea = lookup_widget (window, "drawingarea1");
    create_pixmaps (drawingarea, 0);
    redraw_all_cells (drawingarea);
    preferences_changed = TRUE;
  }
  temp =
    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
				      (initial_level_spinbutton));
  if (temp != initial_level) {
    initial_level = temp;
    preferences_changed = TRUE;
  }
  if (use_graykeys){
     nav_keys[0] = GDK_Right;
     nav_keys[1] = GDK_Left;
     nav_keys[2] = GDK_Up;
     nav_keys[3] = GDK_Down;
     alt_nav_keys[0] = GDK_Right;
     alt_nav_keys[1] = GDK_Left;
     alt_nav_keys[2] = GDK_Up;
     alt_nav_keys[3] = GDK_Down;
  }
  else{
    nav_keys[0] = GDK_KP_6;
    nav_keys[1] = GDK_KP_4;
    nav_keys[2] = GDK_KP_8;
    nav_keys[3] = GDK_KP_5;
    alt_nav_keys[0] = GDK_KP_Right;
    alt_nav_keys[1] = GDK_KP_Left;
    alt_nav_keys[2] = GDK_KP_Up;
    alt_nav_keys[3] = GDK_KP_Begin;
  }
}

void
on_preferences_button_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *dialog, *main_window;

  if (preferences_dialog_shown)
    return;
  preferences_dialog_shown = TRUE;
  main_window = lookup_widget (GTK_WIDGET (button), "main_window");
  dialog = create_preferences_dialog ();
  gtk_object_set_data (GTK_OBJECT (dialog), MainWindowKey, main_window);
  gtk_widget_show (dialog);
}

void
on_main_window_draw (GtkWidget * widget,
		     GdkRectangle * area, gpointer user_data)
{
  GtkWidget *drawingarea;

  drawingarea = lookup_widget (GTK_WIDGET (widget), "drawingarea1");
  redraw_all_cells (widget);
}


gboolean
on_drawingarea1_expose_event (GtkWidget * widget,
			      GdkEventExpose * event, gpointer user_data)
{
  redraw_all_cells (widget);
  return FALSE;
}

void
on_numkeys_radiobutton_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
use_graykeys = 0;
preferences_changed = TRUE;
}


void
on_graykeys_radiobutton_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
use_graykeys = 1;
preferences_changed = TRUE;
}

