/* GtkTetcolor
 * Copyright (C) 1999 Andrey V. Panov
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <sys/stat.h>
#include <gdk/gdkkeysyms.h>

#include "interface.h"
#include "support.h"
#include "main.h"
#include "callbacks.h"
#include "score.h"

gint stop;
gint cell_width = 28, initial_level = 1, max_score = 0, use_graykeys = 0;
gint nav_keys[4], alt_nav_keys[4];

GdkPixmap *colors[NUMBER_COLORS + 1], *border_w_gdkxpm, *border_h_gdkxpm;
GdkBitmap *cell_mask[NUMBER_COLORS + 1], *border_w_mask, *border_h_mask;
gchar *color[NUMBER_COLORS + 1][MAX_CELL_SIZE + 3];
gchar *border_width_xpm[2 + border];
gchar *border_height_xpm[2 + MAX_CELL_SIZE * Y_SIZE];

gboolean create_pixmaps (GtkWidget * widget, gint first_time)
{
  gint i, j;
  gchar cell_size_char[20];
  gchar border_line_width[cell_width * X_SIZE + 2 * border + 1];
  GtkStyle *style;
  gchar cell_core[MAX_CELL_SIZE + 1], cell_border[MAX_CELL_SIZE + 1];
  static gchar *color_name[NUMBER_COLORS + 1] = { "#3F3F3F",
    "blue",
    "green",
    "#00FBE7",
    "#FF00FF",
    "red",
    "yellow",
    "black"
  };


/* prepare strings */
  g_snprintf (cell_size_char, 20, "%d %d 2 1", cell_width, cell_width);
  for (j = 0; j < cell_width; j++) {
    cell_border[j] = '+';
  }
  cell_border[cell_width] = '\0';
  cell_core[0] = '+';
  cell_core[cell_width - 1] = '+';
  cell_core[cell_width] = '\0';
  for (j = 1; j < cell_width - 1; j++)
    cell_core[j] = '@';

/* allocating memory for pixmaps */
  if (first_time) {
    color[0][0] = (gchar *) g_malloc (MAX_CELL_SIZE + 1);
    color[0][1] = (gchar *) g_malloc (MAX_CELL_SIZE + 1);
    color[0][3] = (gchar *) g_malloc (MAX_CELL_SIZE + 1);
    color[1][1] = (gchar *) g_malloc (MAX_CELL_SIZE + 1);
    color[1][4] = (gchar *) g_malloc (MAX_CELL_SIZE + 1);
    for (i = 0; i < NUMBER_COLORS + 1; i++)
      color[i][2] = (gchar *) g_malloc (MAX_CELL_SIZE + 1);
    border_width_xpm[0] = (gchar *) g_malloc (20);
    border_width_xpm[1] = (gchar *) g_malloc (20);
    border_height_xpm[0] = (gchar *) g_malloc (20);
    border_width_xpm[2] = (gchar *) g_malloc (MAX_CELL_SIZE * X_SIZE +
					      2 * border + 1);
    border_height_xpm[2] = (gchar *) g_malloc (border + 1);

    strcpy (color[0][1], "       c None");
    strcpy (color[0][2], "+      c #3F3F3F");

    strcpy (color[1][1], "+      c #877687");
    for (i = 2; i < NUMBER_COLORS + 1; i++)
      color[i][1] = color[1][1];
    for (i = 1; i < NUMBER_COLORS + 1; i++) {
      color[i][0] = color[0][0];
      strcpy (color[i][2], "@      c ");
      strcat (color[i][2], color_name[i]);
    }

    strcpy (border_width_xpm[1], ".      c #F4DB42");
    border_height_xpm[1] = border_width_xpm[1];
    strcpy (border_height_xpm[2], "..");
  }
/* background pixmap */
  strcpy (color[0][0], cell_size_char);
  strcpy (color[0][3], cell_border);
  for (i = 4; i < cell_width + 3; i++) {
    color[0][i] = color[0][3];
  }
/* colored pixmaps */
  strcpy (color[1][4], cell_core);
  for (i = 1; i < NUMBER_COLORS + 1; i++) {
    color[i][3] = color[0][3];
    if (i != 1)
      color[i][4] = color[1][4];
    for (j = 5; j < cell_width + 2; j++)
      color[i][j] = color[1][4];
    color[i][cell_width + 2] = color[0][3];
  }
  for (i = 0; i < NUMBER_COLORS + 1; i++)
    for (j = cell_width + 3; j < MAX_CELL_SIZE + 3; j++)
      color[i][j] = NULL;
/* border pixmaps */
  g_snprintf (border_width_xpm[0], 20, "%d %d 1 1",
	      cell_width * X_SIZE + 2 * border, border);
  for (i = 0; i < cell_width * X_SIZE + 2 * border; i++)
    border_line_width[i] = '.';
  border_line_width[cell_width * X_SIZE + 2 * border] = '\0';
  strcpy (border_width_xpm[2], border_line_width);
  border_width_xpm[3] = border_width_xpm[2];

  g_snprintf (border_height_xpm[0], 20, "%d %d 1 1",
	      border, cell_width * Y_SIZE);
  for (i = 3; i < cell_width * Y_SIZE + 2; i++)
    border_height_xpm[i] = border_height_xpm[2];

  style = gtk_widget_get_style (widget);
  for (i = 0; i < NUMBER_COLORS + 1; i++) {
    if (colors[i])
      gdk_pixmap_unref (colors[i]);
    if (cell_mask[i])
      gdk_bitmap_unref (cell_mask[i]);
    colors[i] = gdk_pixmap_create_from_xpm_d (widget->window, &cell_mask[i],
					      &style->bg[GTK_STATE_NORMAL],
					      (gchar **) & color[i]);
  }
  if (border_w_gdkxpm)
    gdk_pixmap_unref (border_w_gdkxpm);
  if (border_w_mask)
    gdk_bitmap_unref (border_w_mask);
  border_w_gdkxpm = gdk_pixmap_create_from_xpm_d (widget->window,
						  &border_w_mask,
						  &style->bg[GTK_STATE_NORMAL],
						  (gchar **) &
						  border_width_xpm);
  if (border_h_gdkxpm)
    gdk_pixmap_unref (border_h_gdkxpm);
  if (border_h_mask)
    gdk_bitmap_unref (border_h_mask);
  border_h_gdkxpm = gdk_pixmap_create_from_xpm_d (widget->window,
						  &border_h_mask,
						  &style->bg[GTK_STATE_NORMAL],
						  (gchar **) &
						  border_height_xpm);
  gtk_widget_set_usize (widget, X_SIZE * cell_width + 2 * border,
			Y_SIZE * cell_width + 2 * border);
/*  gtk_drawing_area_size (GTK_DRAWING_AREA (widget), 
                         X_SIZE * cell_width + 2 * border, 
			 Y_SIZE * cell_width + 2 * border);
 */
  return TRUE;
}


int
main (int argc, char *argv[])
{
  GtkWidget *main_window;
  GtkWidget *drawingarea;
  GdkPixmap *icon_xpm;
  GdkBitmap *mask;
  GtkStyle *style;
  GtkWidget *max_score_number_label;
  gchar max_score_text[10];
#include "gtktetcolor.xpm"

  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (PACKAGE);

  gtk_set_locale ();
  gtk_init (&argc, &argv);

  add_pixmap_directory (PACKAGE_DATA_DIR G_DIR_SEPARATOR_S "pixmaps");
  add_pixmap_directory (PACKAGE_SOURCE_DIR G_DIR_SEPARATOR_S "pixmaps");

  main_window = create_main_window ();
  gtk_widget_show (main_window);

  drawingarea = lookup_widget (main_window, "drawingarea1");

  choose_score_file ();
  load_preferences ();
  if (use_graykeys){
     nav_keys[0] = GDK_Right;
     nav_keys[1] = GDK_Left;
     nav_keys[2] = GDK_Up;
     nav_keys[3] = GDK_Down;
     alt_nav_keys[0] = GDK_Right;
     alt_nav_keys[1] = GDK_Left;
     alt_nav_keys[2] = GDK_Up;
     alt_nav_keys[3] = GDK_Down;
  }
  else{
    nav_keys[0] = GDK_KP_6;
    nav_keys[1] = GDK_KP_4;
    nav_keys[2] = GDK_KP_8;
    nav_keys[3] = GDK_KP_5;
    alt_nav_keys[0] = GDK_KP_Right;
    alt_nav_keys[1] = GDK_KP_Left;
    alt_nav_keys[2] = GDK_KP_Up;
    alt_nav_keys[3] = GDK_KP_Begin;
  }
  create_pixmaps (drawingarea, 1);
  style = gtk_widget_get_style (main_window);
  icon_xpm = gdk_pixmap_create_from_xpm_d (main_window->window, &mask,
					   &style->bg[GTK_STATE_NORMAL],
					   (gchar **) & gtktetcolor_xpm);

  gdk_window_set_icon (main_window->window, NULL, icon_xpm, NULL);
  srand (time (0));
  read_score ();
  max_score = saved_score[0];
  max_score_number_label =
    lookup_widget (GTK_WIDGET (main_window), "max_score_number_label");
  g_snprintf (max_score_text, 10, "%d", max_score);
  gtk_label_set_text (GTK_LABEL (max_score_number_label), max_score_text);
  init_game (main_window);

  gtk_main ();
  return 0;
}

void
init_game (GtkWidget * widget)
{
  gchar num[2];
  GtkWidget *drawingarea;
  GtkWidget *window;
  GtkWidget *label;
  GtkWidget *score_label_number, *bonus_label_number, *bonus_label;
  gchar score_text[10];
  gint i, j;

  level = initial_level;
  lines = 0;
  score = 0;
  level_tic = 0;
  continue_game = 1;
  bonus_tic = 0;
  interval = INI_INTERVAL;
  for (j = 0; j < Y_SIZE; j++) {
    for (i = 0; i < X_SIZE; i++) {
      cell_state[i][j] = 0;
      cell_state_wo_block[i][j] = 0;
      removing_cell[i][j] = 0;
    }
  }

  drawingarea = lookup_widget (GTK_WIDGET (widget), "drawingarea1");
  window = lookup_widget (GTK_WIDGET (widget), "main_window");
  label = lookup_widget (GTK_WIDGET (widget), "label_level_number");
  score_label_number =
    lookup_widget (GTK_WIDGET (widget), "score_label_number");
  bonus_label_number =
    lookup_widget (GTK_WIDGET (widget), "bonus_label_number");
  bonus_label = lookup_widget (GTK_WIDGET (widget), "bonus_label");
  redraw_cells (drawingarea);
  if (timeout)
    gtk_timeout_remove (timeout);
  if (level_timeout)
    gtk_timeout_remove (level_timeout);
  level_timeout =
    gtk_timeout_add (LEVEL_INT, (GtkFunction) change_level, window);
  g_snprintf (num, 2, "%d", level);
  gtk_label_set_text (GTK_LABEL (label), num);
  g_snprintf (score_text, 10, "%d", score);
  gtk_label_set_text (GTK_LABEL (score_label_number), score_text);
  gtk_label_set_text (GTK_LABEL (bonus_label_number), "");
  gtk_label_set_text (GTK_LABEL (bonus_label), "");
  new_block (window);
  redraw_cells (drawingarea);
}

gboolean redraw_cells (GtkWidget * drawingarea)
{
  gint i, j, cell_state_;

  for (j = 0; j < Y_SIZE; j++) {
    for (i = 0; i < X_SIZE; i++) {
      cell_state_ = cell_state[i][j];
      if (cell_state_prev[i][j] != cell_state_) {
	gdk_draw_pixmap (drawingarea->window,
			 drawingarea->style->
			 fg_gc[GTK_WIDGET_STATE (drawingarea)],
			 colors[cell_state_], 0, 0, border + i * cell_width,
			 border + j * cell_width, cell_width, cell_width);
	cell_state_prev[i][j] = cell_state_;
      }
    }
  }
  return FALSE;
}

gboolean redraw_all_cells (GtkWidget * drawingarea)
{
  gint i, j;

  for (i = 0; i < 2; i++)
    gdk_draw_pixmap (drawingarea->window,
		     drawingarea->style->fg_gc[GTK_WIDGET_STATE (drawingarea)],
		     border_w_gdkxpm, 0, 0, 0,
		     i * (border + Y_SIZE * cell_width),
		     cell_width * X_SIZE + 2 * border, border);
  for (i = 0; i < 2; i++)
    gdk_draw_pixmap (drawingarea->window,
		     drawingarea->style->fg_gc[GTK_WIDGET_STATE (drawingarea)],
		     border_h_gdkxpm, 0, 0, i * (X_SIZE * cell_width + border),
		     border, border, cell_width * Y_SIZE);
  for (j = 0; j < Y_SIZE; j++) {
    for (i = 0; i < X_SIZE; i++) {
      gdk_draw_pixmap (drawingarea->window,
		       drawingarea->style->
		       fg_gc[GTK_WIDGET_STATE (drawingarea)],
		       colors[cell_state[i][j]], 0, 0, border + i * cell_width,
		       border + j * cell_width, cell_width, cell_width);
    }
  }
  return FALSE;
}

gboolean new_block (GtkWidget * main_window)
{
  gint i;

  interval = INI_INTERVAL - INI_INTERVAL * (level - 1) / 10;
  timeout = gtk_timeout_add (interval, (GtkFunction) timeout_callback,
			     main_window);
  if (cell_state_wo_block[3][0]) {
    game_over (main_window);
    return FALSE;
  }
  block.type = rand () % NUMBER_TYPES;
/* Initial location of new block */
  block.x[0] = 3;
  block.y[0] = 0;
  if (block.type) {
    if (cell_state_wo_block[3][1]) {
      game_over (main_window);
      return FALSE;
    }
    block.x[1] = 3;
    block.y[1] = 1;
    switch (block.type) {
    case 1:
      block.num = 2;
      break;
    case 2:
      if (cell_state_wo_block[3][2]) {
	game_over (main_window);
	return FALSE;
      }
      block.x[2] = 3;
      block.y[2] = 2;
      block.num = 3;
      break;
    case 3:
      if (cell_state_wo_block[2][1]) {
	game_over (main_window);
	return FALSE;
      }
      block.x[2] = 2;
      block.y[2] = 1;
      block.num = 3;
      break;
    }
  }
  else
    block.num = 1;
  for (i = 0; i < block.num; i++) {
    block.color[i] = (rand () % (NUMBER_COLORS - 1)) + 1;
    cell_state[block.x[i]][block.y[i]] = block.color[i];
  }
  return TRUE;
}

gboolean block_land (void)
{
  int i, j;
  for (i = 0; i < block.num; i++)
    if (cell_state_wo_block[block.x[i]][block.y[i] + 1] || block.y[i] + 1 >= Y_SIZE) {	/* cannot move down */
      for (j = 0; j < block.num; j++)
	cell_state_wo_block[block.x[j]][block.y[j]] = block.color[j];
      return FALSE;
    }
  for (i = 0; i < block.num; i++)
    cell_state[block.x[i]][block.y[i]] = 0;
  for (i = 0; i < block.num; i++) {
    block.y[i]++;
    cell_state[block.x[i]][block.y[i]] = block.color[i];
  }
  return TRUE;
}

void
game_over (GtkWidget * widget)
{
  GtkWidget *bonus_label;

  if (timeout)
    gtk_timeout_remove (timeout);
  if (level_timeout)
    gtk_timeout_remove (level_timeout);
  continue_game = 0;
  bonus_label = lookup_widget (GTK_WIDGET (widget), "bonus_label");
  gtk_label_set_text (GTK_LABEL (bonus_label), "Game over");
  insert_new_score ();
}

GtkFunction timeout_callback (GtkWidget * window)
{
  GtkWidget *drawingarea;
  GtkWidget *statusbar;
  GtkWidget *score_label_number, *bonus_label_number;
  GtkWidget *max_score_number_label, *bonus_label;
  gchar score_text[10], bonus_text[10], status_text[20], max_score_text[10];

  drawingarea = lookup_widget (GTK_WIDGET (window), "drawingarea1");
  score_label_number = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (window),
                                                       "score_label_number");
  max_score_number_label =  (GtkWidget*) gtk_object_get_data (GTK_OBJECT (window),
                                                   "max_score_number_label");
  if (block_land ()) {
    redraw_cells (drawingarea);
  }
  else {
    gtk_timeout_remove (timeout);
    stop = 0;
    check_lines (window);
    redraw_cells (drawingarea);
    max_score = max_score > score ? max_score : score;
    g_snprintf (score_text, 10, "%d", score);
    g_snprintf (max_score_text, 10, "%d", max_score);
    gtk_label_set_text (GTK_LABEL (score_label_number), score_text);
    gtk_label_set_text (GTK_LABEL (max_score_number_label), max_score_text);
    statusbar = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (window), "statusbar1");
    gtk_statusbar_pop (GTK_STATUSBAR (statusbar), 1);
    strcpy (status_text, "Score ");
    strcat (status_text, score_text);
    gtk_statusbar_push (GTK_STATUSBAR (statusbar), 1, status_text);
    if (bonus) {
      bonus_tic = 0;
      g_snprintf (bonus_text, 10, "%d", bonus);
      bonus_label_number = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (window),
                                                         "bonus_label_number");
      bonus_label = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (window),
                                                         "bonus_label");
      gtk_label_set_text (GTK_LABEL (bonus_label_number), bonus_text);
      gtk_label_set_text (GTK_LABEL (bonus_label), "bonus");
    }
    new_block (window);
    redraw_cells (drawingarea);
/*    return FALSE; */
  }
  return TRUE;
}

GtkFunction change_level (GtkWidget * window)
{
  gchar num[2];
  GtkWidget *label;
  GtkWidget *bonus_label_number, *bonus_label;

  if (!(++level_tic % 60) && level < 9) {
    label = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (window),
                                              "label_level_number");
    level++;
    g_snprintf (num, 2, "%d", level);
    gtk_label_set_text (GTK_LABEL (label), num);
  }
  if (++bonus_tic >= 5) {
    bonus_label_number = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (window),
                                                         "bonus_label_number");
    bonus_label = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (window),
                                                         "bonus_label");
    gtk_label_set_text (GTK_LABEL (bonus_label_number), "");
    gtk_label_set_text (GTK_LABEL (bonus_label), "");
    bonus_tic = 0;
  }
  return TRUE;
}

gboolean shift_block (gint direction)
{
  gint newx, i;
  for (i = 0; i < block.num; i++) {
    newx = block.x[i] + direction;
    if (cell_state_wo_block[newx][block.y[i]] || newx >= X_SIZE || newx < 0)
      return FALSE;
  }
  for (i = 0; i < block.num; i++)
    cell_state[block.x[i]][block.y[i]] = 0;
  for (i = 0; i < block.num; i++) {
    block.x[i] += direction;
    cell_state[block.x[i]][block.y[i]] = block.color[i];
  }
  return TRUE;
}

gboolean rotate_block (rotate_cell)
gboolean (*rotate_cell)(gint, gint *, gint *);
{
  gint newx1, newy1, newx2, newy2;
  switch (block.type) {
  case 1:
    if (!(*rotate_cell)(0, &newx1, &newy1))
      return FALSE;
    cell_state[block.x[0]][block.y[0]] = 0;
    change_rotated_cell (0, newx1, newy1);
    break;
  case 2:
  case 3:
    if (!(*rotate_cell)(0, &newx1, &newy1))
      return FALSE;
    if (!(*rotate_cell)(2, &newx2, &newy2))
      return FALSE;
    cell_state[block.x[0]][block.y[0]] = 0;
    cell_state[block.x[2]][block.y[2]] = 0;
    change_rotated_cell (0, newx1, newy1);
    change_rotated_cell (2, newx2, newy2);
    break;
  }
  return TRUE;
}

gboolean rotate_cell_right (gint i, gint * newx, gint * newy)
{
  *newx = block.y[1] - block.y[i] + block.x[1];
  *newy = block.x[i] - block.x[1] + block.y[1];
  if (cell_state_wo_block[*newx][*newy] || *newx < 0 || *newx >= X_SIZE
      || *newy >= Y_SIZE)
    return FALSE;
  return TRUE;
}

gboolean rotate_cell_left (gint i, gint * newx, gint * newy)
{
  *newx = block.y[i] - block.y[1] + block.x[1];
  *newy = block.x[1] - block.x[i] + block.y[1];
  if (cell_state_wo_block[*newx][*newy] || *newx < 0 || *newx >= X_SIZE
      || *newy >= Y_SIZE)
    return FALSE;
  return TRUE;
}

void
change_rotated_cell (gint i, gint newx, gint newy)
{
  block.x[i] = newx;
  block.y[i] = newy;
  cell_state[block.x[i]][block.y[i]] = block.color[i];
}

gboolean seek_removing_cells (void)
{
  gint i, j, min_y_cell, add_score;
  gboolean return_val = FALSE;

  add_score = MIN_SCORE + MIN_SCORE * (level - 1) / 10;
/* search for vertical lines */
  for (i = 0; i < X_SIZE; i++) {
    min_y_cell = Y_SIZE - 1;
    j = -1;
    do
      j++;
    while (!cell_state[i][j]);
    min_y_cell = j;
    for (j = min_y_cell; j < Y_SIZE - 2; j++) {
      if (cell_state[i][j] == cell_state[i][j + 1]
	  && cell_state[i][j] == cell_state[i][j + 2] && cell_state[i][j]) {
	removing_cell[i][j] = 1;
	removing_cell[i][j + 1] = 1;
	removing_cell[i][j + 2] = 1;
	if (j) {		/* check for 4 cells in line */
	  if (cell_state[i][j] == cell_state[i][j - 1]) {
	    if (j > 1) {	/* check for 5 cells in line */
	      if (cell_state[i][j] == cell_state[i][j - 2]) {
		add_score = add_score * (1.50 / 1.25);
	      }
	      else {
		add_score = add_score + add_score / 4;
	      }
	    }
	    else {
	      add_score = add_score + add_score / 4;
	    }
	  }
	}
	if (j == Y_SIZE - 3 || cell_state[i][j] != cell_state[i][j + 3]) {
	  score += add_score;
	  lines++;
	}
	return_val = TRUE;
      }
    }
  }
/* search for horizontal lines */
  for (j = 0; j < Y_SIZE; j++) {
    for (i = 0; i < X_SIZE - 2; i++) {
      if (cell_state[i][j] && cell_state[i][j] == cell_state[i + 1][j]
	  && cell_state[i][j] == cell_state[i + 2][j]) {
	removing_cell[i][j] = 1;
	removing_cell[i + 1][j] = 1;
	removing_cell[i + 2][j] = 1;
	if (i) {		/* check for 4 cells in line */
	  if (cell_state[i][j] == cell_state[i - 1][j]) {
	    if (i > 1) {	/* check for 5 cells in line */
	      if (cell_state[i][j] == cell_state[i - 2][j]) {
		add_score = add_score * (1.50 / 1.25);
	      }
	      else {
		add_score = add_score + add_score / 4;
	      }
	    }
	    else {
	      add_score = add_score + add_score / 4;
	    }
	  }
	}
	if (i == X_SIZE - 3 || cell_state[i][j] != cell_state[i + 3][j]) {
	  score += add_score;
	  lines++;
	}
	return_val = TRUE;
      }
    }
  }
/* search for diagonal lines */
  for (j = 2; j < Y_SIZE; j++) {
    for (i = 0; i < X_SIZE - 2; i++) {
      if (cell_state[i][j] && cell_state[i][j] == cell_state[i + 1][j - 1]
	  && cell_state[i][j] == cell_state[i + 2][j - 2]) {
	removing_cell[i][j] = 1;
	removing_cell[i + 1][j - 1] = 1;
	removing_cell[i + 2][j - 2] = 1;
	if (i && j < Y_SIZE - 1) {	/* check for 4 cells in line */
	  if (cell_state[i][j] == cell_state[i - 1][j + 1]) {
	    if (i > 1 && j < Y_SIZE - 2) {	/* check for 5 cells in line */
	      if (cell_state[i][j] == cell_state[i - 2][j + 2]) {
		add_score = add_score * (1.50 / 1.25);
	      }
	      else {
		add_score = add_score + add_score / 4;
	      }
	    }
	    else {
	      add_score = add_score + add_score / 4;
	    }
	  }
	}
	if ((i == X_SIZE - 3 && j == 2)
	    || cell_state[i][j] != cell_state[i + 3][j - 3]) {
	  score += add_score;
	  lines++;
	}
	return_val = TRUE;
      }
    }
  }
/* search for backward diagonal lines */
  for (j = 0; j < Y_SIZE - 2; j++) {
    for (i = 0; i < X_SIZE - 2; i++) {
      if (cell_state[i][j] && cell_state[i][j] == cell_state[i + 1][j + 1]
	  && cell_state[i][j] == cell_state[i + 2][j + 2]) {
	removing_cell[i][j] = 1;
	removing_cell[i + 1][j + 1] = 1;
	removing_cell[i + 2][j + 2] = 1;
	if (i && j) {		/* check for 4 cells in line */
	  if (cell_state[i][j] == cell_state[i - 1][j - 1]) {
	    if (i > 1 && j > 1) {	/* check for 5 cells in line */
	      if (cell_state[i][j] == cell_state[i - 2][j - 2]) {
		add_score = add_score * (1.50 / 1.25);
	      }
	      else {
		add_score = add_score + add_score / 4;
	      }
	    }
	    else {
	      add_score = add_score + add_score / 4;
	    }
	  }
	}
	if ((i == X_SIZE - 3 && j == Y_SIZE - 3)
	    || cell_state[i][j] != cell_state[i + 3][j + 3]) {
	  score += add_score;
	  lines++;
	}
	return_val = TRUE;
      }
    }
  }
  return return_val;
}

void
check_lines (GtkWidget *widget)
{
  gint i, j;
  GtkWidget *drawingarea;
  const struct timespec delay_time={0, 50000000};
  struct timespec remain_time;

  lines = 0;
  bonus = 0;
  drawingarea = lookup_widget (GTK_WIDGET (widget), "drawingarea1");
  for (j = 0; j < Y_SIZE; j++) {
    for (i = 0; i < X_SIZE; i++) {
      removing_cell[i][j] = 0;
    }
  }
  while (seek_removing_cells ()) {
  for (j = 0; j < Y_SIZE; j++) {
    for (i = 0; i < X_SIZE; i++) {
      if (removing_cell[i][j]){
      cell_state[i][j] = NUMBER_COLORS;
	}
    }
  }
  redraw_cells (drawingarea); /* Why it does not want to work first time ?*/
  redraw_all_cells (drawingarea);
  nanosleep (&delay_time, &remain_time);
  remove_cells ();
  }
  if (lines > 1) {
    bonus = (lines - 1) * 500;
    score += bonus;
  }
}

gboolean remove_cells (void)
{
  gint i, j, k, prev;
  for (i = 0; i < X_SIZE; i++) {
    for (j = Y_SIZE - 1; j >= 0; j--) {
      if (removing_cell[i][j]) {
	k = j;
	j++;
	while (k >= 0) {
	  removing_cell[i][k] = k ? removing_cell[i][k - 1] : 0;
	  prev = k ? cell_state[i][k - 1] : 0;
	  cell_state[i][k] = prev;
	  cell_state_wo_block[i][k] = prev;
	  if (!prev)
	    goto a;
	  k--;
	}
      a:
      }
    }
  }
  return TRUE;
}

void
load_preferences ()
{
  FILE *rcfile;
  char *str;
  int i, temp;

  str = (gchar *) g_malloc (20);
  rcfile = fopen (rcfile_path, "r");
  if (rcfile == NULL)
    g_print ("cannot open rc file for reading");
  else {
    for (i = 0; !feof (rcfile) && i <= 2; i++) {
      fscanf (rcfile, "%s %d", str, &temp);
      if (strcmp (str, "cell_size") == 0) {
	if (temp >= 10 && temp <= MAX_CELL_SIZE)
	  cell_width = temp;
      }
      else if (strcmp (str, "initial_level") == 0){
	  if (temp >= 1 && temp <= 9)
	    initial_level = temp;
      }
      else if (strcmp (str, "use_graykeys") == 0){
	  if (temp == 1 || temp == 0)
	    use_graykeys = temp;
      }
    }
  }
  g_free (str);
}
